/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lms.vew.daten.DavDatenModell;
import de.bsvrz.sys.funclib.application.StandardApplication;
import de.bsvrz.sys.funclib.application.StandardApplicationRunner;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Zusatzprogramm zum l&ouml;schen aller abgelaufenen RDS-Meldungen.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */
public class AbgelaufeneMeldungenLoeschen implements StandardApplication, MutableSetChangeListener {
	/**
	 * MAIN-Methode.
	 *
	 * @param args
	 *            Argumente
	 */
	public static void main(final String[] args) {
		Thread.setDefaultUncaughtExceptionHandler(new Thread.UncaughtExceptionHandler() {

			@Override
			public void uncaughtException(final Thread t, final Throwable e) {
				Logger.getLogger(getClass().getName()).log(Level.INFO, "*** Application wird beendet ***", e); //$NON-NLS-1$
			}

		});

		StandardApplicationRunner.run(new AbgelaufeneMeldungenLoeschen(), args);
	}

	/**
	 * DebugLogger für Debug-Ausgaben.
	 */
	Debug debug;

	/**
	 * Die Verbindung zum Datenverteiler.
	 */
	ClientDavConnection dav;

	/**
	 * Lock-Objekt.
	 */
	private Object loeschLock;

	/**
	 * Standard-Konstruktor.
	 */
	protected AbgelaufeneMeldungenLoeschen() {
		super();
	}

	/**
	 * &Uuml;berpr&uuml;ft, ob alle notwendigen Parameter f&uuml;r die
	 * Applikation gesetzt sind.
	 *
	 * @throws RDSException
	 *             bei fehlenden notwendigen Paramertern
	 */
	public void checkParameter() throws RDSException {
		// teste notwendige Parameter
		final String[] test = { LMSProperties.PROP_KEY_LMS };

		for (final String element : test) {
			final String testprop = LMSProperties.getInstance().getProperty(element);
			if ((testprop == null) || (testprop.length() == 0)) {
				throw new RDSException("Der Parameter '" + element + "' fehlt!"); //$NON-NLS-1$//$NON-NLS-2$
			}
		}
	}

	@Override
	public void initialize(final ClientDavInterface connection) throws Exception {
		debug = Debug.getLogger();
		dav = (ClientDavConnection) connection;

		try {
			run();
		} catch (final Exception e) {
			LogTools.log(debug, LmsMsg.ErrorAppException, e.getMessage());
		}

		connection.disconnect(false, ""); //$NON-NLS-1$
	}

	/**
	 * Wertet die an die Applikation &uuml;bergebenen Kommandozeilenparameter
	 * aus.<br>
	 * Zus&auml;tzlich zu den Standard-Datenverteiler-Argumenten sind folgende
	 * Argumente m&ouml;glich:
	 * <ul>
	 * <li>-LMS=&lt;Konfigurationsobjekt&gt;, PID oder ID der Landesmeldestelle,
	 * mit der Meldungen ausgetauscht werden sollen</li>
	 * <li>-versandDateiPrefix=&lt;Text&gt;, Legt den Pr&auml;fix der Dateinamen
	 * der an die LMS zu versendenden Dateien fest Standard: vrzrds</li>
	 * <li>-sicherungsVerzeichnis=&lt;Pfad&gt;, Legt das Dateiverzeichnis fest,
	 * in das Meldungsdateien kopiert werden, bei deren Auswertung Fehler
	 * aufgetreten sind. Standard: keine Sicherung</li>
	 * <li>-sicherungsZeit=&lt;Tage&gt;, Anzahl Tage, die Dateien im
	 * Sicherungsverzeichnis gespeichert werden Standard: 5 Tage</li>
	 * </ul>
	 *
	 * @param argumentList
	 *            die Liste der &uuml;bergebenen Argumente
	 * @throws Exception
	 *             bei Ausnahmen
	 * @see de.bsvrz.sys.funclib.application.StandardApplication#parseArguments(de.bsvrz.sys.funclib.commandLineArgs.ArgumentList)
	 */
	@Override
	public void parseArguments(final ArgumentList argumentList) throws Exception {
		final String[] myargs = { LMSProperties.PROP_KEY_LMS };

		for (final String myarg : myargs) {
			if (argumentList.hasArgument(myarg)) {
				LMSProperties.getInstance().setProperty(myarg, argumentList.fetchArgument(myarg).asString());
			}
		}
	}

	/**
	 * Startet die Applikation.
	 *
	 * @throws Exception
	 *             bei Ausnahmen
	 */
	public void run() throws Exception {
		try {
			debug = Debug.getLogger();

			checkParameter();

			// teste, ob das Landesmeldestellen-Objekt existiert
			final String lms = LMSProperties.getInstance().getProperty(LMSProperties.PROP_KEY_LMS);
			if ((lms == null) || (lms.length() == 0)) {
				throw new RDSException("Der Parameter 'Landesmeldestelle' ist nicht gesetzt!");
			}

			final SystemObject lmsobjekt = dav.getDataModel().getObject(lms);
			if (lmsobjekt == null) {
				throw new RDSException("Das Objekt '" + lms + "' kann nicht vom Datenverteiler gelesen werden!");
			}

			DavDatenModell.getInstanz().setVerbindung(dav);

			debug.info("Die Meldungsmenge der Landesmeldestelle '" + lms + "' wird gelesen");

			final MutableSet meldungsmenge = ((ConfigurationObject) lmsobjekt).getMutableSet("RDSMeldungen");
			meldungsmenge.addChangeListener(this);

			debug.info("Die RDS-Meldungen der Landesmeldestelle '" + lms + "' werden gelesen");

			final List<SystemObject> meldungenInMenge = meldungsmenge.getElements();
			debug.info("Anzahl der RDS-Meldungen der Landesmeldestelle '" + lms + "': " + meldungenInMenge.size());

			final List<SystemObject> alleMeldungen = dav.getDataModel().getType("typ.rdsMeldung").getElements();

			debug.info("Gesamtanzahl RDS-Meldungen im DAV: " + alleMeldungen.size());

			final List<SystemObject> loeschenAusMenge = new ArrayList<SystemObject>();

			for (final SystemObject object : meldungenInMenge) {
				if (!object.isValid()) {
					debug.info("Das RDS-Meldungsobjekt '" + object + "' in der Menge '" + meldungsmenge
							+ "' ist ungültig und wird deshalb aus der Menge entfernt");
					loeschenAusMenge.add(object);
				}
			}

			final List<SystemObject> loeschen = new ArrayList<SystemObject>();

			for (final SystemObject object : alleMeldungen) {
				debug.fine("RDS-Meldungsobjekt '" + object + "' wird geprüft!");
				final RDSMeldung meldung = new RDSMeldung(object);
				try {
					meldung.datenAktualisierenDav(dav);
				} catch (final RDSException rdse) {
					// das ist kein Fehler, da nur Daten für bestimmte Aspekte
					// gelesen werden
					debug.info("RDS-Meldungsobjekt '" + object + "': " + rdse.getMessage());
					continue;
				}

				if (!meldung.istZeitGueltig()) {
					// boolean istZeitGueltig = meldung.istZeitGueltig();
					debug.info("RDS-Meldung '" + object + "' ist nicht mehr gültig und wird deshalb gelöscht!");
					loeschen.add(object);

					if (meldungenInMenge.contains(object)) {
						// debug.info("RDS-Meldung '" + object +
						// "' wird aus der Menge entfernt!");
						loeschenAusMenge.add(object);
					}
				}
			}

			if (loeschenAusMenge.size() > 0) {
				debug.info("Meldungsmenge wird bereinigt: " + loeschenAusMenge.size() + " Meldungen werden gelöscht");
				loeschLock = new Object();
				synchronized (loeschLock) {
					try {
						meldungsmenge.remove(loeschenAusMenge.toArray(new SystemObject[loeschenAusMenge.size()]));
						loeschLock.wait(20000);
						debug.info("Meldungsmenge wurde bereinigt");
					} catch (final InterruptedException e) {
						e.printStackTrace();
					}
				}
			}

			if (loeschen.size() > 0) {
				debug.info(loeschen.size() + " RDS-Meldungsobjekte werden invalidiert");
				for (final SystemObject so : loeschen) {
					so.invalidate();
				}
			} else {
				debug.info("Keine zu löschenden RDS-Meldungsobjekte gefunden!");
			}
		} catch (final Exception e) {
			debug.error(e.getMessage());
		}

		LogTools.log(debug, LmsMsg.InfoApplikationBeendet);
	}

	@Override
	public void update(final MutableSet set, final SystemObject[] addedObjects, final SystemObject[] removedObjects) {

		if (removedObjects.length > 0) {
			debug.info("Meldungsmenge aktualisiert: " + removedObjects.length + " Objekt(e) gelöscht");

			if (loeschLock != null) {
				synchronized (loeschLock) {
					loeschLock.notify();
				}
			}
		}
		// // System.out.println("gelöschte Objekte: " + removedObjects);
		// for (SystemObject so : removedObjects) {
		// if (myLock != null && so == myLock) {
		// synchronized (myLock) {
		// myLock.notify();
		// }
		// }
		// }
	}
}
