/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * Repr&auml;sentiert ein bin&auml;res RDS-Infoelement.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSBoolean extends AbstractRDSInfoElement {

	/**
	 * Wert.
	 */
	boolean wert = false;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public RDSBoolean(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		if (davdata.isPlain()) {
			wert = davdata.asUnscaledValue().intValue() == 1 ? true : false;
		} else {
			wert = davdata.getUnscaledValue(davAttributName).intValue() == 1 ? true : false;
		}
	}

	@Override
	public void getXmlDaten(final Node node) throws RDSException {
		wert = Integer.parseInt(node.getTextContent()) == 1 ? true : false;
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		if (davdata.isPlain()) {
			davdata.asUnscaledValue().set(wert ? 1 : 0);
		} else {
			davdata.getUnscaledValue(davAttributName).set(wert ? 1 : 0);
		}
	}

	@Override
	public void setXmlDaten(final Element element) {
		element.setTextContent(wert ? "1" : "0");
	}

}
