/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * Schnittstelle f&uuml;r alle RDS-Informationenselemente.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public interface RDSInfoElement {

	/**
	 * Aktualisiert das Element mit vom DAV empfangenen Daten.
	 *
	 * @param davdata
	 *            von DAV empfangene Daten
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void aktualisieren(Data davdata) throws RDSException;

	/**
	 * Liest die Daten aus dem zugeh&ouml;rige XML-Dokument-Knoten ein.
	 *
	 * @param node
	 *            der zugeh&ouml;rige XML-Dokument-Knoten
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void einlesenXML(Node node) throws RDSException;

	/**
	 * Erzeugt das zugeh&ouml;rige XML-Element.
	 *
	 * @param doc
	 *            Dokument
	 * @param parent
	 *            &uuml;bergeordnetes Element
	 * @return das erzeugte Element
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	Element erzeugeDocumentElement(Document doc, Element parent) throws RDSException;

	/**
	 * Liest die Daten aus einem Datenverteilerdatensatz.
	 *
	 * @param davdata
	 *            Datenverteiler-Daten
	 *
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void getDavDaten(Data davdata) throws RDSException;

	/**
	 * Liest die Daten des Objektes aus einem Knoten.
	 *
	 * @param node
	 *            Dokument-Knoten
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void getXmlDaten(Node node) throws RDSException;

	/**
	 * Test, ob das Objekt einen Handler f&uuml;r einen XML-Dokument-Knoten
	 * besitzt.
	 *
	 * @param node
	 *            ein XML-Dokument-Knoten
	 * @return true, wenn das Objekt einen Handler f&uuml;r einen
	 *         XML-Dokument-Knoten besitzt, sonst false
	 */
	boolean hatNodeHandler(Node node);

	/**
	 * Publiziert die Daten zum Datenverteiler.
	 *
	 * @param davdata
	 *            von DAV empfangene Daten
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void publizieren(Data davdata) throws RDSException;

	/**
	 * Setzt die Daten f&uuml;r den Datenverteiler.
	 *
	 * @param davdata
	 *            Datenverteiler-Daten
	 *
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void setDavDaten(Data davdata) throws RDSException;

	/**
	 * Setzt die XML-Daten eines Elementes aus den Daten des Objektes.
	 *
	 * @param element
	 *            Dokument-Element
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	void setXmlDaten(Element element) throws RDSException;
}
