/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import java.util.List;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * Repr&auml;sentiert die Landeskennung einer RDS-Meldung.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSLandeskennung extends RDSReferenz {

	/**
	 * der Textwert in TIC.
	 */
	private String tictext = "DE";

	/**
	 * der Textwert im DaV.
	 */
	private SystemObject davobject = null;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public RDSLandeskennung(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	/**
	 * Findet die Pid des Landeskennungsobjektes des Datenkataloges zu einer
	 * TIC-Landeskennung.
	 *
	 * @param ticLandeskennung
	 *            Landeskennung aus TIC-Datei
	 * @return PID des zugeh&ouml;rigen Objektes des Datenkataloges
	 * @throws RDSException
	 *             wenn kein Objekt bestimmt werden kann
	 */
	private String findeLandeskennungPid(final String ticLandeskennung) throws RDSException {
		final DataModel modell = DavDatenModell.getInstanz().getDataModel();

		final List<SystemObject> objekte = modell.getType("typ.rdsLandesKennung").getElements();

		final AttributeGroup atg = modell.getAttributeGroup("atg.rdsLandesKennung");

		for (final SystemObject objekt : objekte) {
			final Data d = objekt.getConfigurationData(atg);
			if (d != null) {
				final String kennung = d.getTextValue("Landescode").getText();
				if (kennung.equals(ticLandeskennung)) {
					return objekt.getPid();
				}
			}
		}

		throw new RDSException(
				"Das Landeskennungsobjekt zur TIC-Landeskennung '" + ticLandeskennung + "' kann nicht bestimmt werden");
	}

	/**
	 * Findet die Landeskennung f&uuml;r TIC zu einer DAV-Landeskennung.
	 *
	 * @param landeskennungobjekt
	 *            DAV-Objekt Landeskennung
	 *
	 * @return PID des zugeh&ouml;rigen Objektes des Datenkataloges
	 * @throws RDSException
	 *             bei Ausnahmen
	 */
	private String findeLandeskennungTIC(final SystemObject landeskennungobjekt) throws RDSException {
		if (landeskennungobjekt == null) {
			throw new RDSException("Das Landeskennungsobjekt darf nicht null sein");
		}
		final DataModel modell = DavDatenModell.getInstanz().getDataModel();
		final AttributeGroup atg = modell.getAttributeGroup("atg.rdsLandesKennung");

		final Data d = landeskennungobjekt.getConfigurationData(atg);
		return d.getTextValue("Landescode").getText();
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		davobject = davdata.getReferenceValue(getDavAttributName()).getSystemObject();
	}

	@Override
	public void getXmlDaten(final Node node) {
		tictext = node.getTextContent();
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		davdata.getTextValue(getDavAttributName()).setText(findeLandeskennungPid(tictext));
	}

	@Override
	public void setXmlDaten(final Element element) throws RDSException {
		element.setTextContent(findeLandeskennungTIC(davobject));
	}
}
