/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;

/**
 * Status der Version.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSStatus extends RDSInteger {

	/**
	 * Enumerator RDS-Status.
	 */
	public enum RDS_StatusEN {

		/** unbekannter Status. */
		Unbekannt("Unbekannt", 0),

		/** Modifikation. */
		Modifikation("Modifikation", 2),

		/** Aufhebung. */
		Aufhebung("Aufhebung", 4),

		/** Löschung. */
		Loeschung("Löschung", 5),

		/** Erweiterung. */
		Erweiterung("Erweiterung", 6);

		/**
		 * Gibt eine passende Status-Instanz zur&uuml;ck.
		 *
		 * @param id
		 *            numerischer Wert
		 * @return zum numerischen Wert passende Instanz
		 * @throws RDSException
		 *             bei ung&uuml;ltigem Wert
		 */
		public static RDS_StatusEN getInstance(final int id) throws RDSException {
			switch (id) {
			case 0:
				return Unbekannt;
			case 2:
				return Modifikation;
			case 4:
				return Aufhebung;
			case 5:
				return Loeschung;
			case 6:
				return Erweiterung;
			default:
				throw new RDSException("Kein RDS-Status mit id=" + id + " definiert.");
			}
		}

		/**
		 * textueller Wert.
		 */
		private final String value;

		/**
		 * numerischer Wert.
		 */
		private int id;

		/**
		 * Privater Konstruktor.
		 *
		 * @param value
		 *            textueller Wert
		 * @param id
		 *            numerischer Wert
		 */
		private RDS_StatusEN(final String value, final int id) {
			this.value = value;
			this.id = id;
		}

		/**
		 * Gibt den numerischen Wert zur&uuml;ck.
		 *
		 * @return numerischer Wert des Status
		 */
		public int getId() {
			return id;
		}

		/**
		 * Gibt den textuellen Wert zur&uuml;ck.
		 *
		 * @return textueller Wert des Status
		 */
		public String getValue() {
			return value;
		}
	}

	/**
	 * Status.
	 */
	RDS_StatusEN status = RDS_StatusEN.Unbekannt;

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public RDSStatus(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		super.getDavDaten(davdata);
		status = RDS_StatusEN.getInstance(wert);
	}

	/**
	 * Gibt den Status der Meldung zur&uuml;ck.
	 *
	 * @return Status
	 */
	public RDS_StatusEN getRDSMeldungsStatus() {
		return status;
	}

	@Override
	public void getXmlDaten(final Node node) throws RDSException {
		status = RDS_StatusEN.getInstance(Integer.parseInt(node.getTextContent()));
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		davdata.getUnscaledValue(getDavAttributName()).set(status.getId());
	}

	/**
	 * Setzt den Status der Meldung.
	 *
	 * @param neuerstatus
	 *            neuer Status
	 */
	public void setRDSMeldungsStatus(final RDS_StatusEN neuerstatus) {
		status = neuerstatus;
		wert = status.id;
	}
}
