/*
 * Segment 2 (KEx),  SWE 2.1 LMS
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.kex.lms.vew.daten;

import java.text.ParseException;

import org.w3c.dom.Element;
import org.w3c.dom.Node;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.kex.lms.vew.RDSException;
import de.bsvrz.kex.lms.vew.XMLTools;

/**
 * Repr&auml;sentiert eine RDS-Info Zeitangabe.
 *
 * @author BitCtrl Systems GmbH, Gieseler
 */

public class RDSZeitstempel extends AbstractRDSInfoElement {

	/**
	 * Zeitstempel.
	 */
	long zeitstempel = 1; // 0 ist nicht erlaubt

	/**
	 * Erzeugt ein neues Objekt.
	 *
	 * @param xmlName
	 *            Name des XML-ELementes
	 * @param davName
	 *            Name des DAV-Attributes
	 */
	public RDSZeitstempel(final String xmlName, final String davName) {
		super(xmlName, davName);
	}

	@Override
	public void getDavDaten(final Data davdata) throws RDSException {
		zeitstempel = davdata.getTimeValue(getDavAttributName()).getMillis();
	}

	@Override
	public void getXmlDaten(final Node node) throws RDSException {
		try {
			zeitstempel = XMLTools.getTICDateTime(node.getTextContent());
		} catch (final ParseException e) {
			throw new RDSException("Fehler beim Einlesen des TIC-Zeitstempels '" + node.getTextContent()
			+ "' für Element " + getXmlElementName() + "");
		}
	}

	@Override
	public void setDavDaten(final Data davdata) throws RDSException {
		davdata.getTimeValue(getDavAttributName()).setMillis(zeitstempel);
	}

	@Override
	public void setXmlDaten(final Element element) {
		element.setTextContent(XMLTools.getTICDateTime(zeitstempel));
	}

}
